#!/usr/bin/perl
#
# This script generates a landing site location SPK and topocentric
# FK files for given in "Gaussian" coordinates of the landing site in
# degrees wrt to the ellipsoid of the central body as defined by provided
# PcK file.
#
# Run the script without argument to see usage.
#
# Version 3.1.0 -- BVS/NAIF -- August 19, 2003
#
#   Added command line arguments to specify areocentric coordinates,
#   Mars radii, and full site name to be included into the SPK and FK
#   comments.
#
# Version 3.0.0 -- BVS/NAIF -- February 19, 2003
#
#   Replaced MER-A/B with MER-1/2.
#
# Version 2.1.0 -- BVS/NAIF -- October 1, 2002
#
#   Replaced LOCAL_LEVEL with TOPO in the frame name as MER's
#   LOCAL_LEVEL has +Z down.
#
# Version 2.0.0 -- BVS/NAIF -- November 30, 2001
#
#   Tweaked for MER: added site and alt as inputs and made SPK/FK
#   comments more MER specific.
#
# Version 1.0.0 -- BVS/NAIF -- August 20, 2001
#
$scriptname = "make_mer_ls_spk_fk.pl";
$version    = "3.0.0 -- BVS/NAIF -- February 19, 2003";

#
# We need only one executable -- PINPOINT
#
$pinpoint = "/home/bsemenov/misc/programs/pinpoint/pinpoint";

#
#  Get command line arguments if such were provided. Use magic
#  getopts function discovered by Scott :-)
#
use Getopt::Std;
getopts('n:b:c:l:f:p:s:r:g:t:a:d:i:e:h', \%cloptions);

#
#  If help was requested, print it and exit.
#
if ( ! ( defined( %cloptions ) ) || defined( $cloptions{"h"} ) ) {

   print <<"EndOfUsageText";

$scriptname -- Ver. $version

Usage: > $scriptname [options]

where options (all required) are:

inputs:  -n s/c name (example MER-1 or MER-2 )
         -b s/c ID (example: -253 or -254)
         -l landing site id (ex: TM20B2)
         -e landing site name (ex: Gusev Crater)
         -c center ID (ex: 499)
         -f frame name (ex: IAU_MARS)
         -g landing site areodetic longitude, deg
         -t landing site areodetic latitude, deg
         -a landing site altitude, km
         -p input PcK file name
         -d areocentric coords for comments (ex: '3392.33, 175.300, -14.59')
         -i Mars radii for comments (ex: '3396.19, 3396.19, 3376.20' )

outputs: -s output SPK file name
         -r output FK name

EndOfUsageText

   exit(1);

}

#
# Print greetings.
#
print <<"EndOfReportHeadText";

$scriptname -- Ver. $version

Running with the following "raw" command line options:

EndOfReportHeadText

#
# Report command line options 'as is'..
#
foreach $item ( keys %cloptions ) {
   print "-$item '$cloptions{$item}' ";
}
print "\n\n";

#
# Process command line options.
#
if ( defined( $cloptions{"n"} ) ) {
  $scname = $cloptions{"n"};
  $scname =~ tr/a-z/A-Z/;
  $scname =~ s/ /_/g;
} else {
  die "ERROR: $scriptname: no s/c name was provided with option -n.\n\n";
}

if ( defined( $cloptions{"b"} ) ) {
  $scid = $cloptions{"b"};
} else {
  die "ERROR: $scriptname: no s/c ID was provided with option -b.\n\n";
}

if ( defined( $cloptions{"l"} ) ) {
  $site = $cloptions{"l"};
} else {
  die "ERROR: $scriptname: no site id was provided with option -l.\n\n";
}

if ( defined( $cloptions{"e"} ) ) {
  $siten = $cloptions{"e"};
} else {
  die "ERROR: $scriptname: no site name was provided with option -e.\n\n";
}

if ( defined( $cloptions{"c"} ) ) {
  $bodyid = $cloptions{"c"};
} else {
  die "ERROR: $scriptname: no body ID was provided with option -c.\n\n";
}

if ( defined( $cloptions{"f"} ) ) {
  $frame = $cloptions{"f"};
} else {
  die "ERROR: $scriptname: no frame ID was provided with option -f.\n\n";
}

if ( defined( $cloptions{"g"} ) ) {
  $longitude = $cloptions{"g"};
} else {
  die "ERROR: $scriptname: no longitude was provided with option -g.\n\n";
}

if ( defined( $cloptions{"t"} ) ) {
  $latitude = $cloptions{"t"};
} else {
  die "ERROR: $scriptname: no latitude was provided with option -t.\n\n";
}

if ( defined( $cloptions{"a"} ) ) {
  $altitude = $cloptions{"a"};
} else {
  die "ERROR: $scriptname: no altitude was provided with option -a.\n\n";
}

if ( defined( $cloptions{"p"} ) ) {
  $pckfile = $cloptions{"p"};
  if ( ! -e $pckfile || ! -r $pckfile ) {
     die "ERROR: $scriptname: PcK file $pckfile provided with option -p does ".
       "not exist or is not readable.\n\n";
  }
} else {
  die "ERROR: $scriptname: no PcK file name was provided with option -p.\n\n";
}

if ( defined( $cloptions{"d"} ) ) {
  $centriccoords = $cloptions{"d"};
} else {
  die "ERROR: $scriptname: no areocentric coordinates were provided with ".
      "option -d.\n\n";
}

if ( defined( $cloptions{"i"} ) ) {
  $marsradii = $cloptions{"i"};
} else {
  die "ERROR: $scriptname: no Mars radii were provided with ".
      "option -i.\n\n";
}

if ( defined( $cloptions{"s"} ) ) {
  $spkfile = $cloptions{"s"};
  if ( -e $spkfile ) {
     die "ERROR: $scriptname: SPK file $spkfile provided with option -s ".
       "already exist.\n\n";
  }
  $pfile = $spkfile.".pinpoint";
  if ( -e $pfile ) {
     die "ERROR: $scriptname: PINPOINT input file $pfile ".
       "already exist.\n\n";
  }
} else {
  die "ERROR: $scriptname: no SPK file name was provided with option -s.\n\n";
}

if ( defined( $cloptions{"r"} ) ) {
  $fkfile = $cloptions{"r"};
  if ( -e $fkfile ) {
     die "ERROR: $scriptname: FK file $fkfile provided with option -r ".
       "already exist.\n\n";
  }
} else {
  die "ERROR: $scriptname: no FK file name was provided with option -r.\n\n";
}

#
# Report parsed setups.
#
print <<ParsedSetupsEnd;

Complete set of processed setups is:

   s/c name .................. $scname
   s/c ID .................... $scid
   site id ................... $site
   site name ................. $siten
   center ID ................. $bodyid
   frame name ................ $frame
   landing site longitude .... $longitude
   landing site latitude ..... $latitude
   landing site altitude ..... $altitude
   input PcK file name ....... $pckfile
   areocentric coords ........ $centriccoords
   mars radii ................ $marsradii
   output SPK file name ...... $spkfile
   output FK name ............ $fkfile

ParsedSetupsEnd

#
# Get date.
#
$date = `date`;
chop( $date );

#
# Set site ID and NAIF ID.
#
$sitename = $scname."_LANDING_SITE";
$siteid   = $scid."900";

#
# Make PINPOINT setup file
#
open ( PINPOINT, ">$pfile" ) || die "ERROR: $scriptname: Cannot open ".
  "PINPOINT file $pfile opened for write access.\n";

print PINPOINT <<EndPINPOINTText;

$scname Target Landing Site $site ($siten) Coordinates SPK file.
===========================================================================

     Created by Boris Semenov, NAIF/JPL, $date


Objects in the Ephemeris
--------------------------------------------------------

     This file contains the $scname target landing site $site ($siten)
     location of the surface of Mars. The landing site NAIF ID and name are
     $siteid and '$sitename'.


Approximate Time Coverage
--------------------------------------------------------

     This file coverage is:

        COVERAGE BEGIN TIME (TDB)     COVERAGE END TIME (TDB)
        -------------------------     ------------------------
        2000 JAN 01 00:00:00.000      2100 JAN 01 00:00:00.000


Site Coordinates
--------------------------------------------------------

     This file contains landing site location defined by with the following
     Gaussian coordinates:

        Lon =  $longitude degrees East
        Lat =  $latitude degrees North
        Alt =  $altitude km

     These Gaussian coordinates correspond to the following areocentric
     coordinates (R, LON, LAT) = ($centriccoords) and Mars radii
     (Re, Re, Rp) = ($marsradii).

     The corresponding XYZ coordinates were computed by PINPOINT for the
     triaxial ellipsoid surface defined by the body $bodyid radii from
     PcK file '$pckfile'.


Pedigree
--------------------------------------------------------

     This file was created using PINPOINT Utility Program.


Contacts
--------------------------------------------------------

     If you have any questions regarding this data contact

        Boris V. Semenov, NAIF/JPL,
        (818)-354-8136,
        Boris.Semenov\@jpl.nasa.gov


Appendix. Inputs to Pinpoint
===========================================================================

        \\begindata

           SITES     = ( 'LS' )

           LS_CENTER = $bodyid
           LS_FRAME  = '$frame'
           LS_IDCODE = $siteid
           LS_LATLON = ( $latitude $longitude $altitude )
           LS_BOUNDS = ( \@2001-JAN-01, \@2100-JAN-01 )

        \\begintext

EndPINPOINTText

close ( PINPOINT ) || die "ERROR: $scriptname: Cannot close PINPOINT file ".
  "$pfile opened for write access.\n";

#
# Run PINPOINT to make SPK.
#
$result = `$pinpoint -def $pfile -spk $spkfile -pck $pckfile 2>&1`;
$status = ( $? >> 8 );
if ( $status ) {
  die "ERROR: $scriptname: PINPOINT failed. Messages were:\n\n$result\n";
} else {
  unlink( $pfile ) || die "ERROR: $scriptname: Cannot delete PINPOINT file ".
  "$pfile.\n";
}

#
# Come up with frame name and ID and make FK file.
#
$llframename = $scname."_TOPO";
$llframeid   = $scid."900";
$neglong     = -$longitude;
$negcolat    = - ( 90.0 - $latitude );

open ( FRAMES, ">$fkfile" ) || die "ERROR: $scriptname: Cannot open ".
  "FK file $fkfile opened for write access.\n";

print FRAMES <<EndFRAMESText;
KPL/FK

$scname Target Site $site ($siten) Topocentric Frame Definition Kernel
===============================================================================

     Created by Boris Semenov, NAIF/JPL, $date


Frames in this File
--------------------------------------------------------

     This frames kernel defines the topocentric frame at the $scname
     landing site $site ($siten):

        frame name .......... $llframename
        frame ID ............ $llframeid
        frame type........... FIXED
        relative to frame ... $frame

     This diagram shows the frame hierarchy:

                "J2000" INERTIAL
              --------------------
                       |
                       | <--pck
                       V
                   "$frame"
               BODY-FIXED ROTATING
              --------------------
                       |
                       |  <--fixed
                       V
                  "$llframename"
              --------------------


$scname Topocentric Frame Definition
-------------------------------------------------------------------------------

     This frame defines the z axis as the normal outward at the landing
     site, the x axis points at local north with the y axis completing
     the right handed frame (points at local west.)

     Orientation of the frame is given relative to the body fixed rotating
     frame '$frame' (x - along the line of zero longitude intersecting
     the equator, z - along the spin axis, y - completing the right hand
     coordinate frame.)

     The transformation from '$llframename' frame to '$frame'
     frame is a 3-2-3 rotation with defined angles as the negative of
     the site longitude, the negative of the site colatitude, 180 degrees.

     The landing site Gaussian longitude and latitude upon which the
     definition is built are:

        Lon = $longitude degrees East
        Lat =  $latitude degrees North

     These Gaussian coordinates correspond to the following areocentric
     coordinates (R, LON, LAT) = ($centriccoords) and Mars radii
     (Re, Re, Rp) = ($marsradii).

     The coordinates specified above are given with respect to the
     '$frame' instance defined by the rotation/shape model from the
     the PCK file '$pckfile'.

     These keywords implement the frame definition.

        \\begindata

         FRAME_$llframename             = $llframeid
	 FRAME_$llframeid\_NAME           = '$llframename'
	 FRAME_$llframeid\_CLASS          =  4
	 FRAME_$llframeid\_CLASS_ID       =  $llframeid
	 FRAME_$llframeid\_CENTER         =  $siteid

	 TKFRAME_$llframeid\_RELATIVE     = '$frame'
	 TKFRAME_$llframeid\_SPEC         = 'ANGLES'
	 TKFRAME_$llframeid\_UNITS        = 'DEGREES'
	 TKFRAME_$llframeid\_AXES         = ( 3, 2, 3 )
	 TKFRAME_$llframeid\_ANGLES       = ( $neglong, $negcolat, 180.000  )

        \\begintext


Landing Site Name to NAIF ID Mapping
--------------------------------------------------------

     This section contains keywords that map the rover and landing site
     names their NAIF IDs:

        \\begindata

            NAIF_BODY_NAME += ( '$scname' )
            NAIF_BODY_CODE += ( $scid )

            NAIF_BODY_NAME += ( '$sitename' )
            NAIF_BODY_CODE += ( $siteid )

        \\begintext


Contacts
--------------------------------------------------------

     If you have any questions regarding this data contact

        Boris V. Semenov, NAIF/JPL,
        (818)-354-8136,
        Boris.Semenov\@jpl.nasa.gov

EndFRAMESText

close ( FRAMES ) || die "ERROR: $scriptname: Cannot close FK file ".
  "$fkfile opened for write access.\n";


#
# All done. Exit.
#
exit;
